##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
# 
# frozen_string_literal: true

module Twilio
  module REST
    class Messaging < Domain
      class V1 < Version
        class SessionContext < InstanceContext
          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class ParticipantList < ListResource
            ##
            # Initialize the ParticipantList
            # @param [Version] version Version that contains the resource
            # @param [String] session_sid The unique id of the
            #   [Session](https://www.twilio.com/docs/sessions/api/session) for this
            #   participant.
            # @return [ParticipantList] ParticipantList
            def initialize(version, session_sid: nil)
              super(version)

              # Path Solution
              @solution = {session_sid: session_sid}
              @uri = "/Sessions/#{@solution[:session_sid]}/Participants"
            end

            ##
            # Retrieve a single page of ParticipantInstance records from the API.
            # Request is executed immediately.
            # @param [String] attributes An optional string metadata field you can use to
            #   store any data you wish. The string value must contain structurally valid JSON
            #   if specified.  **Note** that if the attributes are not set "{}" will be
            #   returned.
            # @param [String] twilio_address The address of the Twilio phone number (or
            #   WhatsApp number, or Messenger Page ID) that the participant is in contact with.
            #   This field, together with user_address, is only null when the participant is
            #   interacting from a Chat endpoint (see the 'identity' field).
            # @param [Time] date_created The date that this resource was created. This is most
            #   often useful when bulk-importing messages from another system, as the provided
            #   value will be trusted and displayed on SDK clients.
            # @param [Time] date_updated The date that this resource was last updated. This is
            #   most often provided when bulk-importing messages from another system, as the
            #   provided value will be trusted and displayed on SDK clients.
            # @param [String] identity A unique string identifier for the session participant
            #   as [Chat User](https://www.twilio.com/docs/api/chat/rest/users). This parameter
            #   is non-null if (and only if) the participant is using the Programmable Chat SDK
            #   to communicate.
            # @param [String] user_address The address of the participant's device, e.g. a
            #   phone number or Messenger ID. Together with the Twilio Address, this determines
            #   a participant uniquely. This field (with twilio_address) is only null when the
            #   participant is interacting from a Chat endpoint (see the 'identity' field).
            # @return [ParticipantInstance] Newly created ParticipantInstance
            def create(attributes: :unset, twilio_address: :unset, date_created: :unset, date_updated: :unset, identity: :unset, user_address: :unset)
              data = Twilio::Values.of({
                  'Identity' => identity,
                  'UserAddress' => user_address,
                  'Attributes' => attributes,
                  'TwilioAddress' => twilio_address,
                  'DateCreated' => Twilio.serialize_iso8601_datetime(date_created),
                  'DateUpdated' => Twilio.serialize_iso8601_datetime(date_updated),
              })

              payload = @version.create(
                  'POST',
                  @uri,
                  data: data
              )

              ParticipantInstance.new(@version, payload, session_sid: @solution[:session_sid], )
            end

            ##
            # Lists ParticipantInstance records from the API as a list.
            # Unlike stream(), this operation is eager and will load `limit` records into
            # memory before returning.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit.  Default is no limit
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records.  If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Array] Array of up to limit results
            def list(limit: nil, page_size: nil)
              self.stream(limit: limit, page_size: page_size).entries
            end

            ##
            # Streams ParticipantInstance records from the API as an Enumerable.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit. Default is no limit.
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records. If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Enumerable] Enumerable that will yield up to limit results
            def stream(limit: nil, page_size: nil)
              limits = @version.read_limits(limit, page_size)

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
            end

            ##
            # When passed a block, yields ParticipantInstance records from the API.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            def each
              limits = @version.read_limits

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page,
                              limit: limits[:limit],
                              page_limit: limits[:page_limit]).each {|x| yield x}
            end

            ##
            # Retrieve a single page of ParticipantInstance records from the API.
            # Request is executed immediately.
            # @param [String] page_token PageToken provided by the API
            # @param [Integer] page_number Page Number, this value is simply for client state
            # @param [Integer] page_size Number of records to return, defaults to 50
            # @return [Page] Page of ParticipantInstance
            def page(page_token: :unset, page_number: :unset, page_size: :unset)
              params = Twilio::Values.of({
                  'PageToken' => page_token,
                  'Page' => page_number,
                  'PageSize' => page_size,
              })
              response = @version.page(
                  'GET',
                  @uri,
                  params
              )
              ParticipantPage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of ParticipantInstance records from the API.
            # Request is executed immediately.
            # @param [String] target_url API-generated URL for the requested results page
            # @return [Page] Page of ParticipantInstance
            def get_page(target_url)
              response = @version.domain.request(
                  'GET',
                  target_url
              )
              ParticipantPage.new(@version, response, @solution)
            end

            ##
            # Provide a user friendly representation
            def to_s
              '#<Twilio.Messaging.V1.ParticipantList>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class ParticipantPage < Page
            ##
            # Initialize the ParticipantPage
            # @param [Version] version Version that contains the resource
            # @param [Response] response Response from the API
            # @param [Hash] solution Path solution for the resource
            # @return [ParticipantPage] ParticipantPage
            def initialize(version, response, solution)
              super(version, response)

              # Path Solution
              @solution = solution
            end

            ##
            # Build an instance of ParticipantInstance
            # @param [Hash] payload Payload response from the API
            # @return [ParticipantInstance] ParticipantInstance
            def get_instance(payload)
              ParticipantInstance.new(@version, payload, session_sid: @solution[:session_sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '<Twilio.Messaging.V1.ParticipantPage>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class ParticipantContext < InstanceContext
            ##
            # Initialize the ParticipantContext
            # @param [Version] version Version that contains the resource
            # @param [String] session_sid The unique id of the
            #   [Session](https://www.twilio.com/docs/sessions/api/session) for this
            #   participant.
            # @param [String] sid A 34 character string that uniquely identifies this
            #   resource.
            # @return [ParticipantContext] ParticipantContext
            def initialize(version, session_sid, sid)
              super(version)

              # Path Solution
              @solution = {session_sid: session_sid, sid: sid, }
              @uri = "/Sessions/#{@solution[:session_sid]}/Participants/#{@solution[:sid]}"
            end

            ##
            # Update the ParticipantInstance
            # @param [String] attributes An optional string metadata field you can use to
            #   store any data you wish. The string value must contain structurally valid JSON
            #   if specified.  **Note** that if the attributes are not set "{}" will be
            #   returned.
            # @param [Time] date_created The date that this resource was created. This is
            #   useful when bulk-importing messages from another system, as the provided value
            #   will be trusted and displayed on SDK clients.
            # @param [Time] date_updated The date that this resource was last updated.
            # @return [ParticipantInstance] Updated ParticipantInstance
            def update(attributes: :unset, date_created: :unset, date_updated: :unset)
              data = Twilio::Values.of({
                  'Attributes' => attributes,
                  'DateCreated' => Twilio.serialize_iso8601_datetime(date_created),
                  'DateUpdated' => Twilio.serialize_iso8601_datetime(date_updated),
              })

              payload = @version.update(
                  'POST',
                  @uri,
                  data: data,
              )

              ParticipantInstance.new(
                  @version,
                  payload,
                  session_sid: @solution[:session_sid],
                  sid: @solution[:sid],
              )
            end

            ##
            # Fetch a ParticipantInstance
            # @return [ParticipantInstance] Fetched ParticipantInstance
            def fetch
              params = Twilio::Values.of({})

              payload = @version.fetch(
                  'GET',
                  @uri,
                  params,
              )

              ParticipantInstance.new(
                  @version,
                  payload,
                  session_sid: @solution[:session_sid],
                  sid: @solution[:sid],
              )
            end

            ##
            # Deletes the ParticipantInstance
            # @return [Boolean] true if delete succeeds, true otherwise
            def delete
              @version.delete('delete', @uri)
            end

            ##
            # Provide a user friendly representation
            def to_s
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Messaging.V1.ParticipantContext #{context}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Messaging.V1.ParticipantContext #{context}>"
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class ParticipantInstance < InstanceResource
            ##
            # Initialize the ParticipantInstance
            # @param [Version] version Version that contains the resource
            # @param [Hash] payload payload that contains response from Twilio
            # @param [String] session_sid The unique id of the
            #   [Session](https://www.twilio.com/docs/sessions/api/session) for this
            #   participant.
            # @param [String] sid A 34 character string that uniquely identifies this
            #   resource.
            # @return [ParticipantInstance] ParticipantInstance
            def initialize(version, payload, session_sid: nil, sid: nil)
              super(version)

              # Marshaled Properties
              @properties = {
                  'account_sid' => payload['account_sid'],
                  'service_sid' => payload['service_sid'],
                  'messaging_service_sid' => payload['messaging_service_sid'],
                  'session_sid' => payload['session_sid'],
                  'sid' => payload['sid'],
                  'identity' => payload['identity'],
                  'twilio_address' => payload['twilio_address'],
                  'user_address' => payload['user_address'],
                  'attributes' => payload['attributes'],
                  'type' => payload['type'],
                  'date_created' => Twilio.deserialize_iso8601_datetime(payload['date_created']),
                  'date_updated' => Twilio.deserialize_iso8601_datetime(payload['date_updated']),
                  'url' => payload['url'],
              }

              # Context
              @instance_context = nil
              @params = {'session_sid' => session_sid, 'sid' => sid || @properties['sid'], }
            end

            ##
            # Generate an instance context for the instance, the context is capable of
            # performing various actions.  All instance actions are proxied to the context
            # @return [ParticipantContext] ParticipantContext for this ParticipantInstance
            def context
              unless @instance_context
                @instance_context = ParticipantContext.new(@version, @params['session_sid'], @params['sid'], )
              end
              @instance_context
            end

            ##
            # @return [String] The unique id of the Account responsible for this session.
            def account_sid
              @properties['account_sid']
            end

            ##
            # @return [String] The unique id of the Chat Service this session belongs to.
            def service_sid
              @properties['service_sid']
            end

            ##
            # @return [String] The unique id of the SMS Service this session belongs to.
            def messaging_service_sid
              @properties['messaging_service_sid']
            end

            ##
            # @return [String] The unique id of the Session for this participant.
            def session_sid
              @properties['session_sid']
            end

            ##
            # @return [String] A 34 character string that uniquely identifies this resource.
            def sid
              @properties['sid']
            end

            ##
            # @return [String] A unique string identifier for the session participant as Chat User.
            def identity
              @properties['identity']
            end

            ##
            # @return [String] The address of the Twilio phone number that the participant is in contact with.
            def twilio_address
              @properties['twilio_address']
            end

            ##
            # @return [String] The address of the participant's device.
            def user_address
              @properties['user_address']
            end

            ##
            # @return [String] An optional string metadata field you can use to store any data you wish.
            def attributes
              @properties['attributes']
            end

            ##
            # @return [participant.ParticipantType] The type of twilio product, participant is a user of.
            def type
              @properties['type']
            end

            ##
            # @return [Time] The date that this resource was created.
            def date_created
              @properties['date_created']
            end

            ##
            # @return [Time] The date that this resource was last updated.
            def date_updated
              @properties['date_updated']
            end

            ##
            # @return [String] An absolute URL for this participant.
            def url
              @properties['url']
            end

            ##
            # Update the ParticipantInstance
            # @param [String] attributes An optional string metadata field you can use to
            #   store any data you wish. The string value must contain structurally valid JSON
            #   if specified.  **Note** that if the attributes are not set "{}" will be
            #   returned.
            # @param [Time] date_created The date that this resource was created. This is
            #   useful when bulk-importing messages from another system, as the provided value
            #   will be trusted and displayed on SDK clients.
            # @param [Time] date_updated The date that this resource was last updated.
            # @return [ParticipantInstance] Updated ParticipantInstance
            def update(attributes: :unset, date_created: :unset, date_updated: :unset)
              context.update(attributes: attributes, date_created: date_created, date_updated: date_updated, )
            end

            ##
            # Fetch a ParticipantInstance
            # @return [ParticipantInstance] Fetched ParticipantInstance
            def fetch
              context.fetch
            end

            ##
            # Deletes the ParticipantInstance
            # @return [Boolean] true if delete succeeds, true otherwise
            def delete
              context.delete
            end

            ##
            # Provide a user friendly representation
            def to_s
              values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Messaging.V1.ParticipantInstance #{values}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Messaging.V1.ParticipantInstance #{values}>"
            end
          end
        end
      end
    end
  end
end