##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
# 
# frozen_string_literal: true

module Twilio
  module REST
    class Preview < Domain
      class TrustedComms < Version
        ##
        # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
        class PhoneCallList < ListResource
          ##
          # Initialize the PhoneCallList
          # @param [Version] version Version that contains the resource
          # @return [PhoneCallList] PhoneCallList
          def initialize(version)
            super(version)

            # Path Solution
            @solution = {}
            @uri = "/Business/PhoneCalls"
          end

          ##
          # Retrieve a single page of PhoneCallInstance records from the API.
          # Request is executed immediately.
          # @param [String] from The originating Phone Number that belongs to the Account,
          #   given in E.164 format (https://en.wikipedia.org/wiki/E.164).
          # @param [String] to The terminating Phone Number, given in E.164 format
          #   (https://en.wikipedia.org/wiki/E.164).
          # @param [String] url The Twiml URL to manage this phone call.
          # @param [String] reason The business reason for this phone call that will appear
          #   in the terminating device's screen.
          # @return [PhoneCallInstance] Newly created PhoneCallInstance
          def create(from: nil, to: nil, url: nil, reason: :unset)
            data = Twilio::Values.of({'From' => from, 'To' => to, 'Url' => url, 'Reason' => reason, })

            payload = @version.create(
                'POST',
                @uri,
                data: data
            )

            PhoneCallInstance.new(@version, payload, )
          end

          ##
          # Provide a user friendly representation
          def to_s
            '#<Twilio.Preview.TrustedComms.PhoneCallList>'
          end
        end

        ##
        # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
        class PhoneCallPage < Page
          ##
          # Initialize the PhoneCallPage
          # @param [Version] version Version that contains the resource
          # @param [Response] response Response from the API
          # @param [Hash] solution Path solution for the resource
          # @return [PhoneCallPage] PhoneCallPage
          def initialize(version, response, solution)
            super(version, response)

            # Path Solution
            @solution = solution
          end

          ##
          # Build an instance of PhoneCallInstance
          # @param [Hash] payload Payload response from the API
          # @return [PhoneCallInstance] PhoneCallInstance
          def get_instance(payload)
            PhoneCallInstance.new(@version, payload, )
          end

          ##
          # Provide a user friendly representation
          def to_s
            '<Twilio.Preview.TrustedComms.PhoneCallPage>'
          end
        end

        ##
        # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
        class PhoneCallInstance < InstanceResource
          ##
          # Initialize the PhoneCallInstance
          # @param [Version] version Version that contains the resource
          # @param [Hash] payload payload that contains response from Twilio
          # @return [PhoneCallInstance] PhoneCallInstance
          def initialize(version, payload)
            super(version)

            # Marshaled Properties
            @properties = {
                'sid' => payload['sid'],
                'account_sid' => payload['account_sid'],
                'from' => payload['from'],
                'to' => payload['to'],
                'reason' => payload['reason'],
                'created_at' => Twilio.deserialize_iso8601_datetime(payload['created_at']),
                'url' => payload['url'],
            }
          end

          ##
          # @return [String] A string that uniquely identifies this Current Call.
          def sid
            @properties['sid']
          end

          ##
          # @return [String] Account Sid.
          def account_sid
            @properties['account_sid']
          end

          ##
          # @return [String] The originating Phone Number
          def from
            @properties['from']
          end

          ##
          # @return [String] The terminating Phone Number
          def to
            @properties['to']
          end

          ##
          # @return [String] The business reason for this phone call
          def reason
            @properties['reason']
          end

          ##
          # @return [Time] The date this Current Call was created
          def created_at
            @properties['created_at']
          end

          ##
          # @return [String] The URL of this resource.
          def url
            @properties['url']
          end

          ##
          # Provide a user friendly representation
          def to_s
            "<Twilio.Preview.TrustedComms.PhoneCallInstance>"
          end

          ##
          # Provide a detailed, user friendly representation
          def inspect
            "<Twilio.Preview.TrustedComms.PhoneCallInstance>"
          end
        end
      end
    end
  end
end