##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
# 
# frozen_string_literal: true

module Twilio
  module REST
    class Serverless < Domain
      class V1 < Version
        class ServiceContext < InstanceContext
          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class BuildList < ListResource
            ##
            # Initialize the BuildList
            # @param [Version] version Version that contains the resource
            # @param [String] service_sid The unique SID identifier of the Service for this
            #   Build.
            # @return [BuildList] BuildList
            def initialize(version, service_sid: nil)
              super(version)

              # Path Solution
              @solution = {service_sid: service_sid}
              @uri = "/Services/#{@solution[:service_sid]}/Builds"
            end

            ##
            # Lists BuildInstance records from the API as a list.
            # Unlike stream(), this operation is eager and will load `limit` records into
            # memory before returning.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit.  Default is no limit
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records.  If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Array] Array of up to limit results
            def list(limit: nil, page_size: nil)
              self.stream(limit: limit, page_size: page_size).entries
            end

            ##
            # Streams BuildInstance records from the API as an Enumerable.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit. Default is no limit.
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records. If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Enumerable] Enumerable that will yield up to limit results
            def stream(limit: nil, page_size: nil)
              limits = @version.read_limits(limit, page_size)

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
            end

            ##
            # When passed a block, yields BuildInstance records from the API.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            def each
              limits = @version.read_limits

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page,
                              limit: limits[:limit],
                              page_limit: limits[:page_limit]).each {|x| yield x}
            end

            ##
            # Retrieve a single page of BuildInstance records from the API.
            # Request is executed immediately.
            # @param [String] page_token PageToken provided by the API
            # @param [Integer] page_number Page Number, this value is simply for client state
            # @param [Integer] page_size Number of records to return, defaults to 50
            # @return [Page] Page of BuildInstance
            def page(page_token: :unset, page_number: :unset, page_size: :unset)
              params = Twilio::Values.of({
                  'PageToken' => page_token,
                  'Page' => page_number,
                  'PageSize' => page_size,
              })
              response = @version.page(
                  'GET',
                  @uri,
                  params
              )
              BuildPage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of BuildInstance records from the API.
            # Request is executed immediately.
            # @param [String] target_url API-generated URL for the requested results page
            # @return [Page] Page of BuildInstance
            def get_page(target_url)
              response = @version.domain.request(
                  'GET',
                  target_url
              )
              BuildPage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of BuildInstance records from the API.
            # Request is executed immediately.
            # @param [String] asset_versions The list of Asset Version Sids that are included
            #   in this Build. Optional.
            # @param [String] function_versions The list of Function Version Sids that are
            #   included in this Build. Optional.
            # @param [String] dependencies The list of Dependencies that are included in this
            #   Build, each described by a `name` and a `version` in a JSON object. Optional.
            # @return [BuildInstance] Newly created BuildInstance
            def create(asset_versions: :unset, function_versions: :unset, dependencies: :unset)
              data = Twilio::Values.of({
                  'AssetVersions' => Twilio.serialize_list(asset_versions) { |e| e },
                  'FunctionVersions' => Twilio.serialize_list(function_versions) { |e| e },
                  'Dependencies' => dependencies,
              })

              payload = @version.create(
                  'POST',
                  @uri,
                  data: data
              )

              BuildInstance.new(@version, payload, service_sid: @solution[:service_sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '#<Twilio.Serverless.V1.BuildList>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class BuildPage < Page
            ##
            # Initialize the BuildPage
            # @param [Version] version Version that contains the resource
            # @param [Response] response Response from the API
            # @param [Hash] solution Path solution for the resource
            # @return [BuildPage] BuildPage
            def initialize(version, response, solution)
              super(version, response)

              # Path Solution
              @solution = solution
            end

            ##
            # Build an instance of BuildInstance
            # @param [Hash] payload Payload response from the API
            # @return [BuildInstance] BuildInstance
            def get_instance(payload)
              BuildInstance.new(@version, payload, service_sid: @solution[:service_sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '<Twilio.Serverless.V1.BuildPage>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class BuildContext < InstanceContext
            ##
            # Initialize the BuildContext
            # @param [Version] version Version that contains the resource
            # @param [String] service_sid The unique SID identifier of the Service for this
            #   Build.
            # @param [String] sid The unique SID identifier of this Build.
            # @return [BuildContext] BuildContext
            def initialize(version, service_sid, sid)
              super(version)

              # Path Solution
              @solution = {service_sid: service_sid, sid: sid, }
              @uri = "/Services/#{@solution[:service_sid]}/Builds/#{@solution[:sid]}"
            end

            ##
            # Fetch a BuildInstance
            # @return [BuildInstance] Fetched BuildInstance
            def fetch
              params = Twilio::Values.of({})

              payload = @version.fetch(
                  'GET',
                  @uri,
                  params,
              )

              BuildInstance.new(@version, payload, service_sid: @solution[:service_sid], sid: @solution[:sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Serverless.V1.BuildContext #{context}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Serverless.V1.BuildContext #{context}>"
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class BuildInstance < InstanceResource
            ##
            # Initialize the BuildInstance
            # @param [Version] version Version that contains the resource
            # @param [Hash] payload payload that contains response from Twilio
            # @param [String] service_sid The unique SID identifier of the Service for this
            #   Build.
            # @param [String] sid The unique SID identifier of this Build.
            # @return [BuildInstance] BuildInstance
            def initialize(version, payload, service_sid: nil, sid: nil)
              super(version)

              # Marshaled Properties
              @properties = {
                  'sid' => payload['sid'],
                  'account_sid' => payload['account_sid'],
                  'service_sid' => payload['service_sid'],
                  'status' => payload['status'],
                  'asset_versions' => payload['asset_versions'],
                  'function_versions' => payload['function_versions'],
                  'dependencies' => payload['dependencies'],
                  'date_created' => Twilio.deserialize_iso8601_datetime(payload['date_created']),
                  'date_updated' => Twilio.deserialize_iso8601_datetime(payload['date_updated']),
                  'url' => payload['url'],
              }

              # Context
              @instance_context = nil
              @params = {'service_sid' => service_sid, 'sid' => sid || @properties['sid'], }
            end

            ##
            # Generate an instance context for the instance, the context is capable of
            # performing various actions.  All instance actions are proxied to the context
            # @return [BuildContext] BuildContext for this BuildInstance
            def context
              unless @instance_context
                @instance_context = BuildContext.new(@version, @params['service_sid'], @params['sid'], )
              end
              @instance_context
            end

            ##
            # @return [String] Build Sid.
            def sid
              @properties['sid']
            end

            ##
            # @return [String] Account Sid.
            def account_sid
              @properties['account_sid']
            end

            ##
            # @return [String] Service Sid.
            def service_sid
              @properties['service_sid']
            end

            ##
            # @return [build.Status] The current state of the Build.
            def status
              @properties['status']
            end

            ##
            # @return [Hash] List of Asset Version Sids.
            def asset_versions
              @properties['asset_versions']
            end

            ##
            # @return [Hash] List of Function Version Sids.
            def function_versions
              @properties['function_versions']
            end

            ##
            # @return [Hash] List of Dependencies.
            def dependencies
              @properties['dependencies']
            end

            ##
            # @return [Time] The date that this Build was created.
            def date_created
              @properties['date_created']
            end

            ##
            # @return [Time] The date that this Build was updated.
            def date_updated
              @properties['date_updated']
            end

            ##
            # @return [String] The URL of this Build.
            def url
              @properties['url']
            end

            ##
            # Fetch a BuildInstance
            # @return [BuildInstance] Fetched BuildInstance
            def fetch
              context.fetch
            end

            ##
            # Provide a user friendly representation
            def to_s
              values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Serverless.V1.BuildInstance #{values}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Serverless.V1.BuildInstance #{values}>"
            end
          end
        end
      end
    end
  end
end